from __future__ import absolute_import, unicode_literals

import os

AUTH_URL = "https://auth.kyper.co/"
AWS_WEB_SERVICE = "http://aws.kyper.co"
KYPER_DB_LOCATION = os.path.join(os.path.expanduser("~"), ".kyper.db")
